//+------------------------------------------------------------------+
//|                                                       MMHull.mq5 |
//|                                                         Morgatte |
//|                                      mailto:Morgatte@hotmail.com |
//+------------------------------------------------------------------+
#property copyright "Morgatte 01.01.2011"
#property link      "mailto:Morgatte@hotmail.com"
#property version   "1.00"
#property indicator_chart_window
#property indicator_buffers 5                                        // Nombre de Buffers ncessaires au calcul et affichage de l'indicateur.
#property indicator_plots   1                                        // Nombre de courbes  afficher

//--- plot MMHull
#property indicator_label1  "MMHull"                                 // Nom de la courbe n1 ( = Buffer n0 voir ci-dessous SetIndexBuffer)
#property indicator_type1   DRAW_LINE                                // Courbe de type Ligne (pas surface pleine, ni chandelier, ou autre...)
#property indicator_color1  Red                                      // Couleur de la courbe.
#property indicator_style1  STYLE_SOLID                              // Style de la courbe (ligne) "continue".
#property indicator_width1  2                                        // Epaisseur de la courbe.
#property indicator_applied_price PRICE_TYPICAL                      // Le tableau price[] contiendra des prix de type Typical.

#include <MovingAverages.mqh>                                        // Inclusion de la bibliothque MovingAverages permettant d'utiliser
                                                                     // la fonction LinearWeightedMAOnBuffer (et d'autres) qui n'est pas
                                                                     // native de MQL5.

//--- input parameters
input int      periode = 8;                                          // Paramtre modifiable au clavier au lancement de ce script

//--- indicator buffers
double         MMHull_Buffer[];                                      // Buffer (tableau) contenant les donnes  afficher en tant que courbe
double         TypicalPrices_Buffer[];                               // 
double         Price1_Buffer[];                                      // Buffers intermdiaires de travail pour le calcul
double         Price2_Buffer[];                                      // du buffer principal
double         TotalClose_Buffer[];                                  // 




//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit() {

   SetIndexBuffer(0,MMHull_Buffer,INDICATOR_DATA);                   // On relie le buffer MMHull_Buffer au tableau n0 lequel servira  afficher la courbe
   SetIndexBuffer(1,TypicalPrices_Buffer, INDICATOR_CALCULATIONS);   // On relie le buffer TypicalPrices_Buffer au tableau n1 lequel servira  faire des calcul intermdiaires 
   SetIndexBuffer(2,Price1_Buffer, INDICATOR_CALCULATIONS);          // idem
   SetIndexBuffer(3,Price2_Buffer, INDICATOR_CALCULATIONS);          // idem
   SetIndexBuffer(4,TotalClose_Buffer, INDICATOR_CALCULATIONS);      // idem
   
   IndicatorSetInteger(INDICATOR_DIGITS,5);                          // Nb de chiffres aprs la virgule dans l'affichage de la valeur  afficher
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,10);                        // La courbe ne sera pas affiche pour les 10 premiers ticks (les 10 plus anciens de l'historique)
   string shortname;
   StringConcatenate(shortname,"Moyenne Mobile de HULL (",periode,")");
   PlotIndexSetString(0,PLOT_LABEL,"MMHull");                        // Attribue un nom  la courbe  afficher dans la fentre principale.
   IndicatorSetString(INDICATOR_SHORTNAME,shortname);                // Attribue un nom  afficher dans la fentre popup spare du graph principal.


   return(0);
}




//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+

int OnCalculate(const int rates_total,        
                const int prev_calculated,    
                const int begin,              // Index de dpart dans la table price[]  partir duquel les donnes sont disponibles.
                const double &price[])        // Table des prix,  partir de laquelle l'indicateur sera calcule.
  {
/*      
int OnCalculate(const int rates_total,       // Taille de la table des prix (Taille de la Table price[])
                const int prev_calculated,   // Nombre de Barres disponibles lors du prcdent appel de cette fonction
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[]) {     
*/
   
   //----------------------------------------------------------------------------------------------------------------------------------//  
   // FORMULE MATHEMATIQUE DE LA MOYENNE MOBILE DE HULL                                                                                //
   //                                                                                                                                  //
   //hullma = Weightedaverage[round(SQRT(periode))]( 2*Weightedaverage[round(periode/2)](close)-Weightedaverage[periode](close))       //
   //            iMA          [ PERIODE0 ]      (    2 *     iMA        [PERIODE1]     (close) -      iMA     [PERIODE2] (close)   )   //
   //            iMA          [ PERIODE0 ]      (    2 *    ---------Price1------------------- - -----------Price2-----------------)   //
   //            iMA          [ PERIODE0 ]      (-------------------------------------- TotalClose --------------------------------)   //
   //                                                                                                                                  //
   //----------------------------------------------------------------------------------------------------------------------------------//





   ResetLastError();                                  // Remise  zro de la valeur "LastError"
   
   if(rates_total < periode) return(0);               // inutile de faire des calculs de Moyenne Mobile (de priode = 8) 
                                                      // si on n'a pas encore suffisament de donnes  traiter.

   ArraySetAsSeries(MMHull_Buffer,true);              // On impose un sens de lecture des donnes dans le tableau contenant la courbe de Hull
                                                      // du plus rcent vers le plus ancien.
                                                      // MMHull_Buffer[0] correspond au tick actuel, alors que
                                                      // MMHull_Buffer[rates_total-1] est le plus ancien des ticks de l'historique. 
/*                                     
   ArraySetAsSeries(TypicalPrices_Buffer,true);       // Inutile de donner un sens de lecture pour le autres Buffers de Calculs
   ArraySetAsSeries(Price1_Buffer,true);              // Car la fonction LinearWeightedMAOnBuffer qui les manipule, 
   ArraySetAsSeries(Price2_Buffer,true);              // traite dj ces buffers comme des sries (flag AS_SERIES)
   ArraySetAsSeries(TotalClose_Buffer,true);          // Comme on vien de le faire pour le buffer MMHull_Buffer.
*/

   int start;
   if (prev_calculated==0) start=begin;
   else start=prev_calculated;                              // On optimise la boucle for. Elle fera le calcul de 0  rates_total la toute
   for (int i = start; i < rates_total; i++) {              // premire fois car prev_calculated vaut toujours 0 au premier appel de OnCalculate()
      //TypicalPrices_Buffer[i] = (high[i]+low[i]+close[i])/3;// Puis ensuite cette boucle reprendra au niveau du dernier tick calcul car
      TypicalPrices_Buffer[i] = price[i];                   // Puis ensuite cette boucle reprendra au niveau du dernier tick calcul car
   }                                                        // prev_calculated vaut toujours le dernier tick disponible.
   
   
   int weigthsum = 1;                                 // 
   double PERIODE1 = MathRound(periode/2);            // 
   LinearWeightedMAOnBuffer(rates_total,              // 
                           prev_calculated,           // 
                           begin+0,                   // index de dpart dans le tableau d'entre o les donnes sont diffrentes de undefined donc disponibles
                           (int)PERIODE1,             // Periode utilise dans la formule mathmatique vue ci-dessus
                           TypicalPrices_Buffer,      // Buffer d'entre contenant les donne  moyenner
                           Price1_Buffer,             // Buffer de sortie rcevant les donnes traites par cette fonction
                           weigthsum);                // pondrage
                           
                           // Alert(Price1_Buffer[2]+" -- "+Price1_Buffer[3]+" -- "+Price1_Buffer[rates_total-1]);
                                 // Ce qu'il faut comprendre c'est que dans la fonction utilise ci-dessus la PERIODE1 = 8/2 = 4
                                 // La moyenne mobile sera donc calcule sur 4 valeurs conscutives donc on ne peut pas faire la moyenne
                                 // avant d'avoir atteint le 4 membre du tableau (TypicalPrices_Buffer existe de [0]  ..... )
                                 // Mais Price_Buffer[3] est le premier membre dont on peut calculer sa Moyenne Mobile, avant c'est pas possible.        
                                              
  double PERIODE2 = periode;
  LinearWeightedMAOnBuffer(rates_total,
                           prev_calculated,
                           begin+0,
                           (int)PERIODE2,
                           TypicalPrices_Buffer,
                           Price2_Buffer,
                           weigthsum);

                            // Alert(Price2_Buffer[6]+" -- "+Price2_Buffer[7]+" -- "+Price2_Buffer[rates_total-1]);
                                 // Idem explication que pour plus haut.
                                 // Le premier membre du tableau Price2_Buffer pouvant tre dfinit est Price2_Buffer[7] 
                                 // car la periode de calcul est de 8
                                    
                                                           
   for (int i = start; i < rates_total; i++) {
      TotalClose_Buffer[i] = 2 * Price1_Buffer[i] - Price2_Buffer[i];
   }                                                    
   
   
   double definedIndexArray = PERIODE1;                                 // Ces 2 lignes ne sont pas tout  fait indispensables mais
   if (definedIndexArray < PERIODE2) definedIndexArray = PERIODE2;      // elles permettent de mettre la MMobile  0 pour les priodes indfinies
   
   double PERIODE0 = MathRound(MathSqrt(periode));
   LinearWeightedMAOnBuffer(rates_total,
                           prev_calculated,
                           definedIndexArray + begin, // ou 0,          // Index du premier membre dfinit dans les tableaux prcdents, on aurait
                           (int)PERIODE0,                                        // pu mettre ce membre  0, mais alors les premires valeurs
                           TotalClose_Buffer,                                    // calcules par cette fonction auraient t incorrectes.
                           MMHull_Buffer,                                        // ici les premiers membres calculs resteront gals  0
                           weigthsum);                                           // pour signifier que la MMobile est indfinie pour les premiers
                                                                                 // membres calculs. MMHull_Buffer[rates_total-7]
   
   //--- rates-total = Valeur de retour, en vue de remplir le membre prev_calculated du prochain appel de la fonction OnCalculate()
   return(rates_total);
  }
//+------------------------------------------------------------------+
